// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#ifndef _LORM_HPP_
#define _LORM_HPP_

#include <iostream>
#include <memory>
#include <boost/signals2.hpp>
#include <Eigen/Core>

/** \mainpage Library for Optimization on Riemannian Manifolds
 *
 *  This C++ library provides a collection of template classes for optimization methods on
 *  Riemannian manifold, together with final implementations for the Euclidean
 *  space R^d, the d-dimensional sphere S^d, and the rotation group SO(n).
 * 
 *  By default the class templates are instantiated explicitly with scalar type
 *  \c double. A user defined scalar type can be set in the CMAKE build system
 *  by the variable USER_SCALAR_TYPE (e.g. by calling "cmake
 *  -DUSER_SCALAR_TYPE=\c float). Note, if this variable is set to something
 *  different than double precision the NFFT support will be disabled.
 *
 **/

/** \namespace LORM
 *
 * \brief Namespace consisting of all symbols provided by the library for
 * optimization on Riemannian manifolds.
 *
 **/

namespace LORM {
    
} // end namespace LORM

#endif // _LORM_HPP_
