// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#ifndef _MANIFOLD_HPP_
#define _MANIFOLD_HPP_

#include "../LORM.hpp"

namespace LORM {

  /** \class Manifold
   *  \brief The abstract base class for manifolds.
   *
   *  \note The class is non-copyable and non-copy-assignable. Hence,
   *  it cannot be passed nor returned by value.
   *
   * 
   **/

  class Manifold {
  public:
    /// Returns a descriptive name of the manifold.
    const std::string & getName() const { return name_ ; };
    /// Returns the dimension of the manifold, which is the number of local coordinates.
    int getDimension() const { return dimension_; };
    /// Returns the dimension of the ambient space, which is the number of Cartesian coordinates.
    int getAmbientSpaceDimension() const { return ambientSpaceDimension_; };
    
  protected:
    /// Constructor.
    explicit Manifold ( const std::string & name, int dimension, int ambientSpaceDimension ) :
      name_(name), dimension_(dimension), ambientSpaceDimension_(ambientSpaceDimension) {};
 
  private:
    /// \note Non-copyable.
    Manifold ( const Manifold & other) :
      name_ (other.name_),
      dimension_ (other.dimension_),
      ambientSpaceDimension_ (other.ambientSpaceDimension_) { };
    /// \note Non-copy-assignable.
    const Manifold& operator= ( const Manifold & ) { return *this; };
    
    const std::string name_;
    const int dimension_;
    const int ambientSpaceDimension_;
  };
     
}// end namespace LORM

#endif // _MANIFOLD_HPP_
