// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#include "ManifoldPointArray.hpp"

using namespace LORM;

template<typename T_scalar>
ManifoldPointArray<T_scalar>::ManifoldPointArray( 
  const std::shared_ptr<const Manifold> & sp_manifold ) : 
  SignalOnChange (),
  sp_manifold_ (sp_manifold),
  coordinates_() 
{}

template<typename T_scalar>
ManifoldPointArray<T_scalar>::ManifoldPointArray( 
  const ManifoldPointArray<T_scalar> & other ) : 
  SignalOnChange (other),
  sp_manifold_ (other.sp_manifold_),
  coordinates_ (other.coordinates_) 
{}

template<typename T_scalar>
const ManifoldPointArray<T_scalar> & ManifoldPointArray<T_scalar>::operator=(
  const ManifoldPointArray<T_scalar> & rhs ) 
{ 
  if ( this != &rhs ) {
    assert( this->sp_manifold_ == rhs.sp_manifold_ ); // allow only assignment for identical manifold-type
    SignalOnChange::operator=(rhs);
    coordinates_ = rhs.coordinates_;
    SignalOnChange::sendSignal();
  }
  return *this; 
}

template<typename T_scalar>
void ManifoldPointArray<T_scalar>::setCoordinates( 
  const Eigen::Array<T_scalar, Eigen::Dynamic, Eigen::Dynamic> & coordinates) 
{
  assert( coordinates.rows() == sp_manifold_->getAmbientSpaceDimension() );
  coordinates_.resize( coordinates.rows(), coordinates.cols());
  for ( int i=0; i< coordinates_.cols() ; ++i) {
    Eigen::Array<T_scalar, Eigen::Dynamic, 1> temp = coordinates.col(i); 
    projectPointOnManifold( &temp ); 
    coordinates_.col(i) = temp;
  }
  SignalOnChange::sendSignal();
}

//Macro for explicit instantiations of ManifoldPointArray<T_scalar>
#define LORM_INSTANTIATE_MANIFOLDPOINTARRAY(T_scalar)	\
  template class ManifoldPointArray<T_scalar>;

#if defined( LORM_INSTANTIATE_SCALAR_TYPE )
  LORM_INSTANTIATE_MANIFOLDPOINTARRAY( LORM_INSTANTIATE_SCALAR_TYPE )
#else
  LORM_INSTANTIATE_MANIFOLDPOINTARRAY(double)
#endif

#undef LORM_INSTANTIATE_MANIFOLDPOINTARRAY
