// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#ifndef _ROTATION_GROUP_TIMES_R_HPP_
#define _ROTATION_GROUP_TIMES_R_HPP_

#include "../Manifold.hpp"

namespace LORM {

  namespace RotationGroup {
    
    /** \class RotationGroupTimesR
     *
     *  \brief Implements the product of the n(n-1)/2-dimensional rotation group
     *  SO(3) with R as Manifold.
     *
     **/
    
    class RotationGroupTimesR : public Manifold {
    public:
      /// Constructor. One must provide the dimension n of the manifold SO(n) times R.
      explicit RotationGroupTimesR ( int n ) :
	Manifold ( "Rotation Group times R", n*(n-1)/2 + 1,  n*n + 1) {};
    };
    
  } // end namespace RotationGroup
    
} // end namespace LORM

#endif // _ROTATION_GROUP_TIMES_R_HPP_
