// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#ifndef _SPHERICAL_COORDINATES_TIMES_R_HPP_
#define _SPHERICAL_COORDINATES_TIMES_R_HPP_

#include "SphereTimesR.hpp"
#include "../ManifoldParameterization.hpp"

namespace LORM {

  namespace Sphere {
    
    /** \class SphericalCoordinatesTimesR
     *
     *  \brief Implements the parameterization of SphereTimesR for d=2 as
     *  ManifoldParameterization.
     *
     **/
    
    template<typename T_scalar>
    class SphericalCoordinatesTimesR : 
      public ManifoldParameterization<T_scalar> {
    public:
      /// Default-Constructor.  An instance of the 3-dimensional manifold S^2 x R is created.
      explicit SphericalCoordinatesTimesR() :
	ManifoldParameterization<T_scalar> ( std::make_shared<const SphereTimesR> (2) ) {};
      
      /// Constructor. One must provide a valid shared pointer to an instance of
      /// SphereTimesR with dimension = 3. 
      explicit SphericalCoordinatesTimesR(const std::shared_ptr<const SphereTimesR> & sp_sphereTimesR) :
      ManifoldParameterization<T_scalar> ( sp_sphereTimesR ) {
	assert( sp_sphereTimesR->getDimension() == 3); };
      
    /// Returns the shared pointer to SphereTimesR, which is the underlying manifold.
      const std::shared_ptr<const SphereTimesR> getSphereTimesR() const { 
	return std::static_pointer_cast<const SphereTimesR> 
	(this->getManifold()); };
      
  private:
      Eigen::Array<T_scalar, Eigen::Dynamic, 1>  computeParameterization( 
	const Eigen::Array<T_scalar, Eigen::Dynamic, 1> & localCoords ) const;		          
      
      Eigen::Array<T_scalar, Eigen::Dynamic, 1> computeInverseParameterization(              
        const Eigen::Array<T_scalar, Eigen::Dynamic, 1> & pointCoords ) const;
      
      Eigen::Matrix<T_scalar, Eigen::Dynamic, Eigen::Dynamic> computeJacobiMatrix( 
        const Eigen::Array<T_scalar, Eigen::Dynamic, 1> & localPointCoords ) const;
      
      Eigen::Matrix<T_scalar, Eigen::Dynamic, Eigen::Dynamic> computeInverseRiemannianMatrix( 
        const Eigen::Array<T_scalar, Eigen::Dynamic, 1> & localPointCoords ) const;  
      
      Eigen::Matrix<T_scalar, Eigen::Dynamic, Eigen::Dynamic> computeChristoffelMatrixLinearCombination(
	const Eigen::Array<T_scalar, Eigen::Dynamic, 1> & localCoefficients,
	const Eigen::Array<T_scalar, Eigen::Dynamic, 1> & localPointCoords ) const;

    };
    
  }  // end namespace Sphere
    
} // end namespace LORM
  
#endif // _SPHERICAL_COORDINATES_TIMES_R_HPP_
