// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#ifndef _NFFT_UTILS_
#define _NFFT_UTILS_

#include "../LORM.hpp"
#include <complex>
#include <string.h>

extern "C" {
#include <nfft3.h> /* NFFT3 header */
}

namespace LORM {

  /** \namespace LORM::NFFT
   *
   * \brief Namespace consisting of nfft wrapper classes which provide an C++
   * interface to the nfft C library
   *
   **/  

  namespace NFFT {
    
    inline fftw_complex & operator<<(fftw_complex & fftw, const std::complex<double> & std) {
      memcpy( & fftw, &std, sizeof( fftw_complex ) );
      return fftw;
    }
    inline std::complex<double> & operator<<(std::complex<double> & std, const fftw_complex & fftw) {
      memcpy( & std, &fftw, sizeof( fftw_complex ) );
      return std;
    }
    inline fftw_complex & operator-=(fftw_complex & fftw,  const std::complex<double> & std) {
      fftw[0] -= real(std);
      fftw[1] -= imag(std);  
      return fftw;
    }
    inline fftw_complex & operator+=(fftw_complex & fftw,  const std::complex<double> & std) {
      fftw[0] += real(std);
      fftw[1] += imag(std);  
      return fftw;
    }
        
  } // end namespace NFFT

} // end namespace LORM

#endif // _NFFT_UTILS_
