// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#include "NFSFTFourierCoefficients.hpp"

using namespace LORM::NFFT;

const NFSFTFourierCoefficients & NFSFTFourierCoefficients::operator+=(
  const NFSFTFourierCoefficients & other )
 {
   int minSize = std::min( fhat_array_.rows(), other.fhat_array_.rows() );
   for( int i=0; i<minSize; ++i)
     fhat_array_(i) += other.fhat_array_(i);
   
   if( fhat_array_.rows() < other.fhat_array_.rows() ) {
     fhat_array_.conservativeResize( other.fhat_array_.rows() );
     for( int i=minSize; i<other.fhat_array_.rows(); ++i )
       fhat_array_(i) = other.fhat_array_(i);
   }	  
   return *this;
 }

const NFSFTFourierCoefficients & NFSFTFourierCoefficients::operator-=(
  const NFSFTFourierCoefficients & other )
{
  int minSize = std::min( fhat_array_.rows(), other.fhat_array_.rows() );
  for( int i=0; i<minSize; ++i)
    fhat_array_(i) -= other.fhat_array_(i);
  
  if( fhat_array_.rows() < other.fhat_array_.rows() ) {
    fhat_array_.conservativeResize( other.fhat_array_.rows() );
    for( int i=minSize; i<other.fhat_array_.rows(); ++i )
      fhat_array_(i) = -other.fhat_array_(i);
  }	  
  return *this;
}

NFSFTFourierCoefficients NFSFTFourierCoefficients::operator* ( const NFSFTFourierCoefficients & other ) const
{
  int maxSize = std::max(  fhat_array_.rows(), other.fhat_array_.rows() );
  NFSFTFourierCoefficients mult( getPolynomialDegreeFromDimension(maxSize) );
  
  int minSize = std::min( fhat_array_.rows(), other.fhat_array_.rows() );
  for( int i=0; i<minSize; ++i)
    mult.fhat_array_(i) = fhat_array_(i) * other.fhat_array_(i);
  
  return mult;
}

std::ostream & LORM::NFFT::operator<< (std::ostream& os, const NFSFTFourierCoefficients & fhat )
{
  return os << fhat.getArray(); 
}
