// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#include "NFSOFTFourierCoefficients.hpp"

using namespace LORM::NFFT;

const NFSOFTFourierCoefficients & NFSOFTFourierCoefficients::operator += ( 
  const NFSOFTFourierCoefficients & other ) 
{
  int minSize = std::min( fhat_array_.rows(), other.fhat_array_.rows() );
  for( int i=0; i<minSize; ++i)
    fhat_array_(i) += other.fhat_array_(i);
  
  if( fhat_array_.rows() < other.fhat_array_.rows() ) {
    fhat_array_.conservativeResize( other.fhat_array_.rows() );
    for( int i=minSize; i<other.fhat_array_.rows(); ++i )
      fhat_array_(i) = other.fhat_array_(i);
  }	  
  return *this;
}

const NFSOFTFourierCoefficients & NFSOFTFourierCoefficients::operator -= ( 
  const NFSOFTFourierCoefficients & other ) 
{
  int minSize = std::min( fhat_array_.rows(), other.fhat_array_.rows() );
  for( int i=0; i<minSize; ++i)
    fhat_array_(i) -= other.fhat_array_(i);
  
  if( fhat_array_.rows() < other.fhat_array_.rows() ) {
    fhat_array_.conservativeResize( other.fhat_array_.rows() );
    for( int i=minSize; i<other.fhat_array_.rows(); ++i )
      fhat_array_(i) = -other.fhat_array_(i);
	}	  
  return *this;
}

NFSOFTFourierCoefficients NFSOFTFourierCoefficients::operator* ( 
  const NFSOFTFourierCoefficients & other ) const 
{
  int maxSize = std::max(  fhat_array_.rows(), other.fhat_array_.rows() );
  NFSOFTFourierCoefficients mult( getPolynomialDegreeFromDimension(maxSize) );
  
  int minSize = std::min( fhat_array_.rows(), other.fhat_array_.rows() );
  for( int i=0; i<minSize; ++i)
    mult.fhat_array_(i) = fhat_array_(i) * other.fhat_array_(i);
  
  return mult;
}

std::ostream & LORM::NFFT::operator<< (std::ostream& os, const NFSOFTFourierCoefficients & fhat ) 
{
  return os << fhat.getArray();
}
