// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#include "PolynomialS2Vectorized.hpp"

using namespace LORM::ObjectiveFunctions;

PolynomialS2Vectorized::PolynomialS2Vectorized( int numberOfPoints, int degree ) :
  ManifoldFunctionSecondOrderParameterized<double, Eigen::Array<double, 1, Eigen::Dynamic> > ( 
    std::make_shared<const Sphere::SphericalCoordinates<double> > () ),
  nfsftPlan_ ( numberOfPoints, degree ),
  fhat_ ( Eigen::ArrayXcd::Random((degree+1)*(degree+1)) ) 
{}

void PolynomialS2Vectorized::computeFxParameterized( 
  Eigen::Array<double, 1, Eigen::Dynamic> * p_fx, 
  const Eigen::Array<double, Eigen::Dynamic, Eigen::Dynamic> & localPointCoordsArray) const
{
  assert( localPointCoordsArray.cols() == nfsftPlan_.getNumberOfPoints() );
  nfsftPlan_.setLocalPointCoordinatesArray( localPointCoordsArray );
  *p_fx = ((Eigen::ArrayXXcd) nfsftPlan_.evalYMatrixMultiplication( fhat_ )).real().transpose();     
}

void PolynomialS2Vectorized::computeNablaFxParameterizedArray( 
  Eigen::Array<double, Eigen::Dynamic,Eigen::Dynamic> * p_nablaFxParameterizedArray,
  const Eigen::Array<double, Eigen::Dynamic, Eigen::Dynamic> & localPointCoordsArray) const
{
  assert( localPointCoordsArray.cols() == nfsftPlan_.getNumberOfPoints() );
  nfsftPlan_.setLocalPointCoordinatesArray( localPointCoordsArray );
  *p_nablaFxParameterizedArray = 
    ((Eigen::ArrayXXcd) nfsftPlan_.evalNablaYMatrixMultiplication( fhat_ )).real();
}

void PolynomialS2Vectorized::computeNabla2FxParameterizedMultiplication( 
  Eigen::Array<double, Eigen::Dynamic,Eigen::Dynamic> * p_nabla2FxParameterizedMultTangentArray, 
  const Eigen::Array<double, Eigen::Dynamic, Eigen::Dynamic> & localPointCoordsArray ) const
{
  assert( localPointCoordsArray.cols() == nfsftPlan_.getNumberOfPoints() );
  nfsftPlan_.setLocalPointCoordinatesArray( localPointCoordsArray );
  Eigen::Array3Xd nabla2FxParameterizedArray = 
    ((Eigen::ArrayXXcd) nfsftPlan_.evalNabla2YMatrixMultiplication( fhat_ )).real();  
      
  for (int i=0; i<nfsftPlan_.getNumberOfPoints(); ++i) {
    Eigen::Array2d temp = p_nabla2FxParameterizedMultTangentArray->col(i);
    (*p_nabla2FxParameterizedMultTangentArray)(0,i) =  
      nabla2FxParameterizedArray(0,i) * temp(0) +  nabla2FxParameterizedArray(1,i) * temp(1);
    (*p_nabla2FxParameterizedMultTangentArray)(1,i) =  
      nabla2FxParameterizedArray(1,i) * temp(0) +  nabla2FxParameterizedArray(2,i) * temp(1);
  }
}
