// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#ifndef _ARMIJO_METHOD_HPP_
#define _ARMIJO_METHOD_HPP_

#include "LineSearchMethod.hpp"

namespace LORM {
  
  /** \class ArmijoMethod
   *
   *  \brief Implements the armijo line search method on Riemannian manifolds as LineSearchMethod.
   *
   */


  template<typename T_scalar>
  class ArmijoMethod : public LineSearchMethod<T_scalar> {
  public:
    /// Default-Constructor.
    explicit ArmijoMethod() : 
      LineSearchMethod<T_scalar> () {};
  private:
    int performLineSearch (ManifoldPointArray<T_scalar> * p_pointArray,
			   ManifoldTangentVectorArray<T_scalar> * p_tangentArray,
			   const ManifoldFunctionSecondOrder<T_scalar> & function,
			   T_scalar initialStepLength) const;
  };

} // end namespace LORM

#endif // _ARMIJO_METHOD_HPP_
