// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#ifndef _ARMIJO_METHOD_VECTORIZED_HPP_
#define _ARMIJO_METHOD_VECTORIZED_HPP_

#include "LineSearchMethodVectorized.hpp"

namespace LORM {
  
  /** \class ArmijoMethodVectorized
   *
   *  \brief Implements the armijo line search method on Riemannian manifolds
   *  with vectorized function evaluations as LineSearchMethodVectorized.
   *
   */


  template<typename T_scalar>
  class ArmijoMethodVectorized : public LineSearchMethodVectorized<T_scalar> {
  public:
    /// Default-Constructor.
    explicit ArmijoMethodVectorized() : 
      LineSearchMethodVectorized<T_scalar> () {};
  private:
    int performLineSearch (
      ManifoldPointArray<T_scalar> * p_pointArray,
      ManifoldTangentVectorArray<T_scalar> * p_tangentArray,
      const ManifoldFunctionSecondOrder<T_scalar, Eigen::Array<T_scalar, 1, Eigen::Dynamic> > & function,
      Eigen::Array<T_scalar, 1, Eigen::Dynamic> initialStepLengthArray) const;
  };

} // end namespace LORM

#endif // _ARMIJO_METHOD_VECTORIZED_HPP_
