// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#ifndef _CONJUGATE_GRADIENT_MEHTOD_HPP_
#define _CONJUGATE_GRADIENT_MEHTOD_HPP_

#include "DescentMethod.hpp"

namespace LORM {

  /** \class ConjugateGradientMethod
   *
   *  \brief Implements the conjugate gradient method on Riemannian manifolds as DescentMethod.
   *
   */

  template <typename T_scalar>
  class ConjugateGradientMethod : public DescentMethod<T_scalar> {
  public:
    /// Constructor. One must provide a valid shared pointer to an instance of LineSearchMethod.
    explicit ConjugateGradientMethod(
      const std::shared_ptr<const LineSearchMethod<T_scalar> > & sp_lineSearch) :
      DescentMethod<T_scalar>(sp_lineSearch) {};
  private:
    int performOptimization ( ManifoldPointArray<T_scalar> * p_pointArray,
			      const ManifoldFunctionSecondOrder<T_scalar> & function ) const;
  };

} // end namespace LORM

#endif // _CONJUGATE_GRADIENT_MEHTOD_HPP_
