// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#ifndef _CONJUGATE_GRADIENT_MEHTOD_VECTORIZED_HPP_
#define _CONJUGATE_GRADIENT_MEHTOD_VECTORIZED_HPP_

#include "DescentMethodVectorized.hpp"

namespace LORM {

  /** \class ConjugateGradientMethodVectorized
   *
   *  \brief Implements the conjugate gradient method on Riemannian manifolds
   *  with vectorized function evaluations as DescentMethodVectorized.
   *
   */

  template <typename T_scalar>
  class ConjugateGradientMethodVectorized : public DescentMethodVectorized<T_scalar> {
  public:
    /// Constructor. One must provide a valid shared pointer to an instance of LineSearchMethodVectorized.
    explicit ConjugateGradientMethodVectorized(
      const std::shared_ptr<const LineSearchMethodVectorized<T_scalar> > & sp_lineSearch) :
      DescentMethodVectorized<T_scalar>(sp_lineSearch) {};
  private:
    int performOptimization ( 
      ManifoldPointArray<T_scalar> * p_pointArray,
      const ManifoldFunctionSecondOrder<T_scalar, Eigen::Array<T_scalar, 1, Eigen::Dynamic> > & function ) const;
  };

} // end namespace LORM

#endif // _CONJUGATE_GRADIENT_MEHTOD_VECTORIZED_HPP_
