// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#include "DescentMethod.hpp"

using namespace LORM;

template<typename T_scalar>
int DescentMethod<T_scalar>::run( 
  ManifoldPointArray<T_scalar> * p_pointArray,
  const ManifoldFunctionSecondOrder<T_scalar> & function ) const
{
  assert( function.getPointArray().get() == p_pointArray );
  return performOptimization( p_pointArray, function );
}

//Macro for explicit instantiations of DescentMethod<T_scalar>
#define LORM_INSTANTIATE_DESCENTMETHOD(T_scalar)	\
  template class DescentMethod<T_scalar>;

#if defined( LORM_INSTANTIATE_SCALAR_TYPE )
  LORM_INSTANTIATE_DESCENTMETHOD( LORM_INSTANTIATE_SCALAR_TYPE )
#else
  LORM_INSTANTIATE_DESCENTMETHOD(double)
#endif

#undef LORM_INSTANTIATE_DESCENTMETHOD
