// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#include "DescentMethodVectorized.hpp"

using namespace LORM;

template<typename T_scalar>
int DescentMethodVectorized<T_scalar>::run(
  ManifoldPointArray<T_scalar> * p_pointArray,
  const ManifoldFunctionSecondOrder<T_scalar, Eigen::Array<T_scalar, 1, Eigen::Dynamic> > & function ) const 
{
  assert( function.getPointArray().get() == p_pointArray );
  return performOptimization( p_pointArray, function );
}

//Macro for explicit instantiations of DescentMethodVectorized<T_scalar>
#define LORM_INSTANTIATE_DESCENTMETHODVECTORIZED(T_scalar)	\
  template class DescentMethodVectorized<T_scalar>;

#if defined( LORM_INSTANTIATE_SCALAR_TYPE )
  LORM_INSTANTIATE_DESCENTMETHODVECTORIZED( LORM_INSTANTIATE_SCALAR_TYPE )
#else
  LORM_INSTANTIATE_DESCENTMETHODVECTORIZED(double)
#endif

#undef LORM_INSTANTIATE_DESCENTMETHODVECTORIZED
