// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#include "LineSearchMethod.hpp"
#include "../Utils/Math.hpp"

using namespace LORM;

template<typename T_scalar>
int LineSearchMethod<T_scalar>::run( 
   ManifoldPointArray<T_scalar> * p_pointArray,
   ManifoldTangentVectorArray<T_scalar> * p_tangentArray,
   const ManifoldFunctionSecondOrder<T_scalar> & function,
   T_scalar initialStepLength) const
{
  assert( function.getPointArray().get() == p_pointArray );
  assert( p_tangentArray->getBasePointArray().get() == p_pointArray);
      
  return performLineSearch( p_pointArray, p_tangentArray, function, initialStepLength);
}

//Macro for explicit instantiations of LineSearchMethod<T_scalar>
#define LORM_INSTANTIATE_LINESEARCHMETHOD(T_scalar)	\
  template class LineSearchMethod<T_scalar>;

#if defined( LORM_INSTANTIATE_SCALAR_TYPE )
  LORM_INSTANTIATE_LINESEARCHMETHOD( LORM_INSTANTIATE_SCALAR_TYPE )
#else
  LORM_INSTANTIATE_LINESEARCHMETHOD(double)
#endif

#undef LORM_INSTANTIATE_LINESEARCHMETHOD
