// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#include "LineSearchMethodVectorized.hpp"

using namespace LORM;

template<typename T_scalar>
int LineSearchMethodVectorized<T_scalar>::run(
  ManifoldPointArray<T_scalar> * p_pointArray,
  ManifoldTangentVectorArray<T_scalar> * p_tangentArray,
  const ManifoldFunctionSecondOrder<T_scalar, Eigen::Array<T_scalar, 1, Eigen::Dynamic> > & function,
  Eigen::Array<T_scalar, 1, Eigen::Dynamic> initialStepLengthArray) const
{
  assert( function.getPointArray().get() == p_pointArray );
  assert( p_tangentArray->getBasePointArray().get() == p_pointArray);
  assert( initialStepLengthArray.cols() == p_pointArray->getCoordinates().cols() );
      
  auto f0 = function.evalFunction();
  
  return performLineSearch( p_pointArray, p_tangentArray, function, initialStepLengthArray);
}

//Macro for explicit instantiations of LineSearchMethodVectorized<T_scalar>
#define LORM_INSTANTIATE_LINESEARCHMETHODVECTORIZED(T_scalar)	\
  template class LineSearchMethodVectorized<T_scalar>;

#if defined( LORM_INSTANTIATE_SCALAR_TYPE )
  LORM_INSTANTIATE_LINESEARCHMETHODVECTORIZED( LORM_INSTANTIATE_SCALAR_TYPE )
#else
  LORM_INSTANTIATE_LINESEARCHMETHODVECTORIZED(double)
#endif

#undef LORM_INSTANTIATE_LINESEARCHMETHODVECTORIZED
