// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#ifndef _LORM_MATH_HPP_
#define _LORM_MATH_HPP_

#include "../LORM.hpp"

namespace LORM {

  /** \namespace LORM::Math
   *
   * \brief Namespace consisting of general math related helper functions and templates.
   *
   **/

  namespace Math {
    
    /// Returns the absolute value of a real-valued number t.
    template<typename T_scalar>
    T_scalar realAbs( T_scalar t ) {
      return (t >= T_scalar(0) ? t : -t);
    } 

    /// Returns the integer part of a real-valued number t.
    template<typename T_integer, typename T_scalar>
    T_integer realFloor( T_scalar t ) {
      return t < 0 ? (T_integer) t == t ? (T_integer) t : (T_integer) t -1 : (T_integer) t;
    }
    
    /// Returns the fractional part of n modulo a given denominater d.
    template<typename T_scalar>
    T_scalar realMod( T_scalar n, T_scalar d) {
      return n - realFloor<int, T_scalar>(n/d) * d;
    }   

  } // end namespace math
} // end namespace LORM

#endif // _LORM_MATH_HPP_
