// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#ifndef _TEST_UTILS_HPP_
#define _TEST_UTILS_HPP_

#include "../ObjectiveFunctions/ManifoldFunctionSecondOrder.hpp"
#include <iostream>
#include <iomanip>
#include <fstream>


namespace LORM {

  template <typename T_scalar>
  void evalLineFunctionWithQuadraticApproximation(
          const ManifoldFunctionSecondOrder<T_scalar> & function,
	  const ManifoldPointArray<T_scalar> & pointArray,
	  const ManifoldTangentVectorArray<T_scalar> & tangentArray,
	  T_scalar stepLength, 
	  int sampleSize,
	  const char * filename );

  template <typename T_scalar>
  void evalLineFunctionVectorizedWithQuadraticApproximation(
    const ManifoldFunctionSecondOrder<T_scalar, Eigen::Array<T_scalar, 1, Eigen::Dynamic> >& function,
    const ManifoldPointArray<T_scalar> & pointArray,
    const ManifoldTangentVectorArray<T_scalar> & tangentArray,
    T_scalar stepLength, 
    int sampleSize,
    const char * filename );

} // end namespace LORM

#endif // _TEST_UTILS_HPP_
