// This file is part of LORM, a  C++ library for optimization on 
// Riemannian manifolds.
//
// Copyright (C) 2013 Manuel Gräf <m.graef@mathematik.tu-chemnitz.de>
//
// This source code form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#include "../src/NFFTwrapper/NFFT2d.hpp"

/** \file test_nfft2d.cpp
   *
   *  \brief A simple example demonstrating the functionality of the 2-dimensional nfft
   *  wrapper class LORM::NFFT::NFFT2d.
   *
   */


int main () {

  using namespace std;
  using namespace LORM;
  using namespace NFFT;
  
  int M = 3; // number of points
  Eigen::Array2i degrees; // polynomial degrees
  degrees << 4,1;

  // initialize random points in [2*pi, 2*pi)^2
  Eigen::ArrayXXd pointsLocal = M_PI * (Eigen::ArrayXXd::Random(2,M) + 1);

  // creates new instance of an NFFT2d plan with polynomial degrees (N1,N2) and
  // number of points M
  NFFT2d nfft2dPlan(M,degrees);  
  // sets the point coordinates
  nfft2dPlan.setLocalPointCoordinatesArray( pointsLocal );
  
  cout << "Initialized NFFT2d plan with polynomial degrees N=" << endl 
       << degrees << endl
       << " and number of Points M=" << endl << M << endl;
  cout << " nfft local point coordinats: " << endl 
       << nfft2dPlan.getLocalPointCoordinatesArray() << endl << endl;

  // creates a new instance of NFFT2d Fourier coefficients
  NFFT2dFourierCoefficients fhat( (Eigen::ArrayXXcd) 
	  Eigen::ArrayXXcd::Random( 2*degrees(0), 2*degrees(1) ) );
  
  // prints the transformations provided by the NFFT2d plan
  cout << "Random Fourier coefficients, fhat: " << endl 
       << fhat << endl;
  cout << "nfft 2d Trafo (Y * fhat):" << endl
       << nfft2dPlan.evalYMatrixMultiplication( fhat ) << endl;
  cout << "nfft 2d Nabla Trafo ( Nabla Y * fhat ):" << endl
       << nfft2dPlan.evalNablaYMatrixMultiplication( fhat ) << endl;
  cout << "nfft 2d Nabla^2 Trafo( Nabla^2 Y * fhat ):" << endl
       << nfft2dPlan.evalNabla2YMatrixMultiplication( fhat ) << endl << endl;

  // creats an array of function values
  Eigen::ArrayXcd f = Eigen::ArrayXcd::Random(M);
  // prints the adjoint transformations provided by the NFFT2d plan
  cout << "Random (complex) Function values, f:" << endl << f << endl;
  cout << "nfft 2d Adjoint Trafo (Y' * f): " << endl 
       << nfft2dPlan.evalYMatrixAdjointMultiplication( f ) << endl << endl;
  // creats an array of nabla function values
  Eigen::Array2Xd f2 = Eigen::Array2Xd::Random(2,M);
  cout << "Random (real) Nabla values, f2:" << endl << f2 << endl;
  cout << "nfft 2d Adjoint Nabla Trafo ( (Nabla Y)' * f2 ):" << endl
       << nfft2dPlan.evalNablaYMatrixAdjointMultiplication( f2 ) << endl;

  return 0;
}
